/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: PISync.h$
 * $Date: Sun Aug 10 06:29:38 1997$
 *
 Description:
	Definition of synchronization objects and methods.
\*____________________________________________________________________________*/
/* $HeaderTop:$ */

#ifndef PISYNC_H_
#define PISYNC_H_

#include "PiAPI.h"

/*____________________________________________________________________________*\
 *
 Typedefs:
\*____________________________________________________________________________*/
typedef struct Semaphore PISync;

/*____________________________________________________________________________*\
 *
 Name:
	PISync_lock

 Synopsis:
	int PISync_lock( PISync *pSync )
	
 Description:
	Attempts to grap a mutex resource or increment the count on a 
	semaphore resource. If the lock cannot be obtained PISync_lock() will
	block until it is obtained.

 Notes:
	The behaviour of PISync_lock depends on the type of the synchronization
	object pSync. 

	PISync_delete() should be used to destroy a synchronization object when
	it is no longer needed.

 Return Values:
	PISync_lock returns zero (PIAPI_COMPLETED) on success.

 Errors:
	PIAPI_ERROR			an error occurred.

	PIPlatform_getLastError() can be used to obtain more detailed
	error information.

 See Also:
	PISync_lock(), PISync_tryLock(), PISync_unlock().
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PISync_lock( PISync *pSync );

/*____________________________________________________________________________*\
 *
 Name:
	PISync_tryLock

 Synopsis:
	int PISync_tryLock( PISync *pSync )
	
 Description:
	Attempts to grap a mutex resource or increment the count on a 
	semaphore resource. If the lock cannot be immediately obtained
	PISync_lock() will return without blocking.

 Notes:
	The behaviour of PISync_tryLock depends on the type of the synchronization
	object pSync. 

	PISync_delete() should be used to destroy a synchronization object when
	it is no longer needed.

 Return Values:
	PISync_lock returns zero (PIAPI_COMPLETED) if the lock was obtained or
	PIAPI_WOULDBLOCK if the lock could not be obtained.

 Errors:
	PIAPI_ERROR			an error occurred.

	PIPlatform_getLastError() can be used to obtain more detailed error
	information.

 See Also:
	PISync_lock(), PISync_tryLock(), PISync_unlock().
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PISync_tryLock( PISync *pSync );

/*____________________________________________________________________________*\
 *
 Name:
	PISync_unlock

 Synopsis:
	int PISync_unlock( PISync *pSync )
	
 Description:
	Free's a lock that was placed on a mutex synchronization object or
	increments the free count of a semaphore synchronization object. This
	function removes the lock placed by PISync_lock() or PISync_tryLock().

 Notes:
	The behaviour of PISync_unlock depends on the type of the synchronization
	object pSync. 

	PISync_delete() should be used to destroy a synchronization object when
	it is no longer needed.

 Return Values:
	PISync_unlock returns zero (PIAPI_COMPLETED) on success.

 Errors:
	PIAPI_ERROR			an error occurred.

	PIPlatform_getLastError() can be used to obtain more detailed error
	information.

 See Also:
	PISync_lock(), PISync_tryLock(), PISync_unlock().
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PISync_unlock( PISync *pSync );

/*____________________________________________________________________________*\
 *
 Name:
	PISync_delete

 Synopsis:
	int PISync_delete( PISync *pSync )
	
 Description:
	Destroy's a synchronization object. 

 Notes:
 Return Values:
	PISync_delete returns zero (PIAPI_COMPLETED) on success.

 Errors:
	PIAPI_ERROR			an error occurred.

	PIPlatform_getLastError() can be used to obtain more detailed error
	information.

 See Also:
	PIPlatform_allocLocalMutex(), PIPlatform_allocLocalSemaphore().
\*____________________________________________________________________________*/
PUBLIC_PIAPI int PISync_delete( PISync *pSync );

#endif	/* PISYNC_H_ */

